#include "zgscore.h"
/* Yes I am a C geezer! */
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "sys/stat.h"

static int current_focus = PLAYER1;
static int current_hole = 1;
static int scores[NUM_PLAYERS][NUM_HOLES];
static QLineEdit *n[NUM_PLAYERS];
static QLCDNumber *h, *s[NUM_PLAYERS];
static QLCDNumber *a[NUM_PLAYERS];
static QLCDNumber *b[NUM_PLAYERS];
static QLCDNumber *t[NUM_PLAYERS];

static void update_all_values() {
  int inx, jnx, sum1, sum2;

  h->display(current_hole);

  for( inx=0; inx<NUM_PLAYERS; inx++ ) {
    s[inx]->display(scores[inx][current_hole-1]);
    sum1 = 0;
    for( jnx=0; jnx<NUM_HOLES/2; jnx++ ) {
      sum1 += scores[inx][jnx];
    }
    a[inx]->display(sum1);
    sum2 = 0;
    for( jnx=NUM_HOLES/2; jnx<NUM_HOLES; jnx++ ) {
      sum2 += scores[inx][jnx];
    }
    b[inx]->display(sum2);
    t[inx]->display(sum1+sum2);
  }
}

static void do_focus() {
  QPalette p;
  int inx;

  /* reset other backgrounds */
  for( inx=0; inx<NUM_PLAYERS; inx++ ) {
    if( inx != current_focus ) {
      p = n[inx]->palette();
      p.setColor( QPalette::Normal, QColorGroup::Base, QColor(255, 255, 255) );
      p.setColor( QPalette::Normal, QColorGroup::Text, QColor(0, 0, 0) );
      n[inx]->setPalette(p);
    }
    else {
      p = n[inx]->palette();
      p.setColor( QPalette::Normal, QColorGroup::Base, QColor(200, 50, 50) );
      p.setColor( QPalette::Normal, QColorGroup::Text, QColor(255, 255, 255) );
      n[inx]->setPalette(p);
      n[inx]->setFocus();
    }
  }
}

bool ZGS::eventFilter( QObject *, QEvent *event )
{
    QKeyEvent *keyEvent;
    bool eat_event = FALSE;
    int key;

    if( event->type() == QEvent::KeyPress ) {
      /* eat all keys unless allowed */
      eat_event = TRUE;
      keyEvent = (QKeyEvent *)event;
      key = keyEvent->key();

      if( (key == Key_Backspace) || (key == Key_Delete) ||
          (key == Key_Shift) || (key == Key_CapsLock) ||
          (key == Key_NumLock) || (key == Key_Home ) ) {
        /* don't eat these! */
        eat_event = FALSE;
      }
      if( (key >= Key_0) && (key <= Key_9) ) {
        /* ascii 0 thru 9 - don't eat me! */
        eat_event = FALSE;
      }
      else if( (key >= Key_A) && (key <= Key_Z) ) {
        /* ascii A thru Z - don't eat me! */
        eat_event = FALSE;
      }
      else if( (key >= Key_A+Key_At) && (key <= Key_Z+Key_At) ) {
        /* ascii a thru z - don't eat me! */
        eat_event = FALSE;
      }
      else if( key == Key_Down ) {
        /* go back one hole! */
        current_hole--;
        if( current_hole < 1 ) current_hole = 1;
        update_all_values();
      }
      else if( key == Key_Up ) {
        /* go forward one hole! */
        current_hole++;
        if( current_hole > NUM_HOLES ) current_hole = NUM_HOLES;
        update_all_values();
      }
      else if( key == Key_Right ) {
        /* increase one stroke! */
        switch( current_focus ) {
          case PLAYER1:
            scores[PLAYER1][current_hole-1]++;
            s[PLAYER1]->display(scores[PLAYER1][current_hole-1]);
            break;
          case PLAYER2:
            scores[PLAYER2][current_hole-1]++;
            s[PLAYER2]->display(scores[PLAYER2][current_hole-1]);
            break;
          case PLAYER3:
            scores[PLAYER3][current_hole-1]++;
            s[PLAYER3]->display(scores[PLAYER3][current_hole-1]);
            break;
          case PLAYER4:
            scores[PLAYER4][current_hole-1]++;
            s[PLAYER4]->display(scores[PLAYER4][current_hole-1]);
            break;
        }
        update_all_values();
      }
      else if( key == Key_Left ) {
        /* decrease one stroke! */
        switch( current_focus ) {
          case PLAYER1:
            scores[PLAYER1][current_hole-1]--;
            if( scores[PLAYER1][current_hole-1] < 0 )
              scores[PLAYER1][current_hole-1] = 0;
            s[PLAYER1]->display(scores[PLAYER1][current_hole-1]);
            break;
          case PLAYER2:
            scores[PLAYER2][current_hole-1]--;
            if( scores[PLAYER2][current_hole-1] < 0 )
              scores[PLAYER2][current_hole-1] = 0;
            s[PLAYER2]->display(scores[PLAYER2][current_hole-1]);
            break;
          case PLAYER3:
            scores[PLAYER3][current_hole-1]--;
            if( scores[PLAYER3][current_hole-1] < 0 )
              scores[PLAYER3][current_hole-1] = 0;
            s[PLAYER3]->display(scores[PLAYER3][current_hole-1]);
            break;
          case PLAYER4:
            scores[PLAYER4][current_hole-1]--;
            if( scores[PLAYER4][current_hole-1] < 0 )
              scores[PLAYER4][current_hole-1] = 0;
            s[PLAYER4]->display(scores[PLAYER4][current_hole-1]);
            break;
        }
        update_all_values();
      }
      else if( (key == Key_Space) || (key == Key_Tab) ) {
        /* switch player! */
        switch( current_focus ) {
          case PLAYER1:
            current_focus = PLAYER2;
            do_focus();
            break;
          case PLAYER2:
            current_focus = PLAYER3;
            do_focus();
            break;
          case PLAYER3:
            current_focus = PLAYER4;
            do_focus();
            break;
          case PLAYER4:
            current_focus = PLAYER1;
            do_focus();
            break;
        }
      }
    }
    return eat_event;
}

/* 
 *  Constructs a ZGS which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 */
ZGS::ZGS( QWidget* parent,  const char* name, WFlags fl )
    : ZGSBase( parent, name, fl )
{
    connect(help, SIGNAL(clicked()), this, SLOT(Help()));
    connect(print, SIGNAL(clicked()), this, SLOT(Print()));

    /* save widget ids for use by other functions...kludge alert! */
    n[0] = name1;
    n[1] = name2;
    n[2] = name3;
    n[3] = name4;
    s[0] = score1;
    s[1] = score2;
    s[2] = score3;
    s[3] = score4;
    h = hole;
    a[0] = tot1_front9;
    a[1] = tot2_front9;
    a[2] = tot3_front9;
    a[3] = tot4_front9;
    b[0] = tot1_back9;
    b[1] = tot2_back9;
    b[2] = tot3_back9;
    b[3] = tot4_back9;
    t[0] = tot1_18;
    t[1] = tot2_18;
    t[2] = tot3_18;
    t[3] = tot4_18;

    /* special keystroke handler on all LineEdit widgets! */
    n[0]->installEventFilter(this);
    n[1]->installEventFilter(this);
    n[2]->installEventFilter(this);
    n[3]->installEventFilter(this);

    /* set current focus */
    current_focus = PLAYER1;
    do_focus();

    /* set current hole */
    current_hole = 1;
    update_all_values();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ZGS::~ZGS()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 *  Help...
 */

void ZGS::Help()
{
  QMessageBox::information( this, "ZGScore - Help",
			    "Zaurus Golf Score Program\n\n"
			    "Only use rocker buttons!\n"
			    "Center Button: Switch Player\n"
			    "Left/Right: Stroke Down/Up\n"
			    "Down/Up: Hole Down/Up\n"
			    "Help Button: Show this help\n"
			    "Print Button: Write a report\n"
			    "Cancel Button: Exit\n"
			    "Change names using keypad!\n\n"
			    "License: GPL\n\n"
			    "Written by David Brown\n"
			    "<db@cinlug.org>." );
}

/*
 *  Print...
 */

void ZGS::Print()
{
  /* I should really learn C++ file i/o, but... */
  FILE *fp;
  int inx, jnx, sum1, sum2, tot;
  char *envbuf;
  char file[64];

  /* save data to file */
  if ((envbuf = getenv("HOME")) != NULL) {
    sprintf( file, "%s/Documents", envbuf );
    mkdir(file, S_IRWXU | S_IRWXG | S_IRWXO );
    sprintf( file, "%s/Documents/text", envbuf );
    mkdir(file, S_IRWXU | S_IRWXG | S_IRWXO );
    sprintf( file, "%s/Documents/text/plain", envbuf );
    mkdir(file, S_IRWXU | S_IRWXG | S_IRWXO );
    sprintf( file, "%s/Documents/text/plain/zgscore.txt", envbuf );
    if( (fp = fopen(file,"w")) ) {

      /* header */
      fprintf( fp, "Score F9 B9 01/02/03/04/05/06/07/08/09/10/11/12/13/14/15/16/17/18/ Name\n" );
      fprintf( fp, "----- -- -- ------------------------------------------------------ ----\n" );

      /* scores */
      for( inx=0; inx<NUM_PLAYERS; inx++ ) {
        sum1 = 0;
        for( jnx=0; jnx<NUM_HOLES/2; jnx++ ) {
          sum1 += scores[inx][jnx];
        }
        sum2 = 0;
        for( jnx=NUM_HOLES/2; jnx<NUM_HOLES; jnx++ ) {
          sum2 += scores[inx][jnx];
        }
        tot = sum1 + sum2;
        if( tot > 0 ) { /* I played, so print me! */
          fprintf( fp, "%5d %2d %2d ", tot, sum1, sum2 );
          for( jnx=0; jnx<NUM_HOLES; jnx++ ) {
            fprintf( fp, "%2d/", scores[inx][jnx] );
          }
          fprintf( fp, " %s\n\n", n[inx]->text().ascii() );
        }
      } 
      fclose(fp);
      QMessageBox::information( this, "ZGScore - Print", "Report printed to: zgscore.txt" );
    }
    else {
      QMessageBox::information( this, "ZGScore - Error", "Report failed to print!" );
    }
  }
}

void ZGS::closeEvent( QCloseEvent* ce )
{
    switch( QMessageBox::information( this, "ZGScore - Confirmation",
                                          "Quit scoring now?",
                                          "Yes", "No", 0, 1, 1 ) ) {
        case 0:
            ce->accept();
            break;
        default:
            ce->ignore();
            break;
    }

}
